#!/usr/bin/python
# -*- coding: utf-8 -*-

import faulthandler
import functools
import unittest

from PyQt5 import QtCore


class CRunnable(QtCore.QObject, QtCore.QRunnable):
    started = QtCore.pyqtSignal(object)
    completed = QtCore.pyqtSignal(object)
    error = QtCore.pyqtSignal(object, Exception)

    def __init__(self, packaged_task):
        QtCore.QObject.__init__(self)
        QtCore.QRunnable.__init__(self)

        self.task = packaged_task

    def autoDelete(self):
        return False

    @QtCore.pyqtSlot()
    def run(self):
        print("Run")
        self.started.emit(self.task)
        try:
            self.task()
        except Exception as e:
            self.error.emit(self.task, e)
            print("Error")
        else:
            self.completed.emit(self.task, e)
            print("End")

faulthandler.enable()
app = QtCore.QCoreApplication([])

class TestCRunnable(unittest.TestCase):
    def setUp(self):
        self.started = False
        self.completed = False

    def test_1_(self):
        pool = QtCore.QThreadPool.globalInstance()

        r2 = CRunnable(functools.partial(QtCore.QThread.msleep, 200))
        r2.started.connect(self._on_started)
        r2.completed.connect(self._on_completed)
        self.assertFalse(self.started)
        self.assertFalse(self.completed)
        pool.start(r2)
        app.exec_()
        self.assertTrue(self.started)
        self.assertTrue(self.completed)

    def _on_started(self, sender):
        print("Started")
        self.started = True
        QtCore.QTimer.singleShot(300, app.quit)

    def _on_completed(self, sender):
        print("Completed")
        self.completed = True


if __name__ == '__main__':
    unittest.main(verbosity=10)
